// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component containing elements to set the instrumental function parameters (used to smooth the spectra to emulate spectra recorded with an instrument)
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: instrumentFunctionItem
    anchors.fill: parent

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        anchors.fill: instrumentFunctionColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: instrumentFunctionItem.top
        anchors.right: instrumentFunctionItem.right
        anchors.margins: coreObj.defaultLabelHeight * 0.1
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            specResultsQml.updatePlotAllPreserveScale();
            instrumentFunctionItem.destroy();
        }
    }

    CustomLabel {
        id: widthLabel
        text: qsTr("Smooth spectrum with an instrument function")
        visible: false
        enabled: false
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        id: instrumentFunctionColumnLayout
        anchors.centerIn: parent
        spacing: 5

        CustomLabel {
            id: commentLabel        
            text: qsTr("Smooth spectrum with an instrument function")
        }

        // Instrument function
        // 0 - None
        // 1 - Rectangular (Boxcar)
        // 2 - Triangular
        // 3 - Gaussian
        // 4 - Dispersion (Lorentz)
        // 5 - Diffraction
        // 6 - Michelson
        // 7 - Cosine
        // ComboBox to select the instrumental function
        CustomComboBox {
            id: instrumentalFunctionComboBox
            Layout.fillWidth: true
            model: [ "None", "Rectangular (Boxcar)", "Triangular", "Gaussian", "Dispersion (Lorentz)", "Diffraction", "Michelson", "Cosine"]
            currentIndex: instrFunctQml.instrFunctIndex
            onCurrentIndexChanged: {
                instrFunctQml.instrFunctIndex = currentIndex
                remoteControlQml.broadcastCommand("INSTRFUNCT", 0)
            }
        }

        GridLayout {

            id: instrResolGridLayout

            // Label for the instrument function resolution, cm-1
            CustomLabel {
                id: instrResolLabel
                anchors.right: instrResolTextField.left
                anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 0
                Layout.row: 1                
                horizontalAlignment: Text.AlignRight
                text: qsTr("Instrument\n resolution, g, (cm-1):");
            }
            // Text field to specifty the instrumental resolution, cm-1
            CustomTextField {
                id: instrResolTextField
                Layout.column: 1
                Layout.row: 1
                text: instrFunctQml.instrResol
                implicitWidth: widthLabel.width - instrResolLabel.width
                onEditingFinished: {
                    instrFunctQml.instrResol = text
                    remoteControlQml.broadcastCommand("INSTRRES", 0)
                }
            }

            // Label for the instrument function wing, cm-1
            CustomLabel {
                id: instrFunctWingLabel
                anchors.right: instrFunctWingTextField.left
                anchors.rightMargin: coreObj.defaultLabelHeight * 0.3
                Layout.column: 0
                Layout.row: 2                
                horizontalAlignment: Text.AlignRight
                text: qsTr("Instrument function\nwing, (cm-1):");
            }
            // Text field to specify the instrumental function wing, cm-1
            CustomTextField {
                id: instrFunctWingTextField
                Layout.column: 1
                Layout.row: 2
                text: instrFunctQml.instrFunctWing
                implicitWidth: widthLabel.width - instrResolLabel.width
                onEditingFinished: {
                    instrFunctQml.instrFunctWing = text
                    remoteControlQml.broadcastCommand("INSTRWING", 0)
                }
            }
        }

        // This portion of code is to display an exaplanation of what the instrument resolution is for different instrument functions selected (taken directly from HAPI source code)
        Rectangle {
            id: separator2Rect
            Layout.fillWidth: true
            Layout.margins: instrResolCommentLabel.height * 0.2
            height: 1
            color: '#D3D3D3'
            visible: {
                if ((instrFunctQml.instrFunctIndex == 0) || (instrFunctQml.instrFunctIndex == 5)) { return false; }
                else { return true; }
            }
        }
        RowLayout {
            anchors.left: instrumentFunctionColumnLayout.left
            anchors.right: instrumentFunctionColumnLayout.right
            Layout.column: 0
            Layout.row: 1
            CustomLabel {
                id: instrResolCommentLabel
                anchors.centerIn: parent
                text: {
                    if (instrFunctQml.instrFunctIndex == 0) { return qsTr(""); }
                    else if (instrFunctQml.instrFunctIndex == 1) { return qsTr("g - slit width or the instrument resolution"); }
                    else if (instrFunctQml.instrFunctIndex == 2) { return qsTr("g - line width == half base of the triangle"); }
                    else if (instrFunctQml.instrFunctIndex == 3) { return qsTr("g/2 - Gaussian half-width at half-maximum"); }
                    else if (instrFunctQml.instrFunctIndex == 4) { return qsTr("g/2 - Lorentzian half-width at half-maximum"); }
                    else if (instrFunctQml.instrFunctIndex == 5) { return qsTr(""); }
                    else if (instrFunctQml.instrFunctIndex == 6) { return qsTr("1/g - the maximum optical path difference"); }
                }
                visible: {
                    if ((instrFunctQml.instrFunctIndex == 0) || (instrFunctQml.instrFunctIndex == 5)) { return false; }
                    else { return true; }
                }
            }
        }
    }
}
