// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component to change molecules parameters
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import QtQuick.Window 2.3

Item {

    property var unitsIndex

    id: moleculesItem
    anchors.fill: parent

    // Adjusting the window size when the phone is rotated
    onHeightChanged:
    {
        if (Screen.primaryOrientation === Qt.PortraitOrientation)
        {
            moleculesColumnLayout.height = moleculesItem.height - exitButton.height - 1.35 * coreObj.defaultLabelHeight - exitButton.padding
            moleculesColumnLayout.anchors.top = exitButton.bottom
            titleLabel.text = "Concentration may be specified in %, parts per\nmillion (ppmv), or as partial pressure"
            unitsIndex = molListViewModel.units
            unitsComboBox.model = [ "Concentration,\n(%)", "Parts per\nmillion, (ppmv)", "Atmospheres,\n(atm)", "Millibar,\n(mbar)", "Torr,\n(Torr)", "Pascal,\n(Pa)", "Pounds per\nsquare inch, psi"]
            unitsComboBox.currentIndex = unitsIndex
            h2oSwitch.rotation = 0
            h2oSwitchGridLayout.flow = GridLayout.LeftToRight
            h2oSwitchGridLayout.anchors.leftMargin = coreObj.defaultLabelHeight * (0.85)
            h2oSwitchGridLayout.anchors.bottomMargin = 0
            manualEntryLabel.Layout.bottomMargin = 0
            sensorDataLabel.Layout.topMargin = 0
        }
        else
        {
            moleculesColumnLayout.height = moleculesItem.height - 1.35 * coreObj.defaultLabelHeight
            moleculesColumnLayout.anchors.top = moleculesItem.top
            titleLabel.text = "Concentration may be specified in percent, parts \nper million (ppmv), or as partial pressure"
            unitsIndex = molListViewModel.units
            unitsComboBox.model = [  "Concentration, (%)", "Parts per million,\n(ppmv)", "Atmospheres, (atm)", "Millibar, (mbar)", "Torr, (Torr)", "Pascal, (Pa)", "Pounds per square\ninch, (psi)"]
            unitsComboBox.currentIndex = unitsIndex
            h2oSwitch.rotation = 90
            h2oSwitchGridLayout.flow = GridLayout.TopToBottom
            h2oSwitchGridLayout.anchors.leftMargin = - h2oSwitchGridLayout.width
            h2oSwitchGridLayout.anchors.bottomMargin = - h2oSwitchGridLayout.height - coreObj.defaultLabelHeight * (2.45)
            manualEntryLabel.Layout.bottomMargin = coreObj.defaultLabelHeight * 0.8
            sensorDataLabel.Layout.topMargin = coreObj.defaultLabelHeight * 0.8
        }
    }

    // Rectangle to shade the background
    Rectangle {
        id: backRect
        anchors.fill: parent
        border.width: 1
        border.color: "black"
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: frameRect
        anchors.fill: moleculesColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        anchors.bottomMargin: coreObj.defaultLabelHeight * (-0.40)
        color: "white"
        border.width: 1
        border.color: "gray"
        radius: coreObj.defaultLabelHeight * 0.5
    }

    CustomTextField {
        id: refTextField
        visible: false
        enabled: false
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: moleculesItem.top
        anchors.right: moleculesItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        Layout.row: 0
        Layout.column: 1
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            molListViewReducedModel.updateDisplayed();
            specResultsQml.updatePlotAllPreserveScale();
            moleculesItem.destroy();
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: h2oSwitchFrameRect
        anchors.fill: h2oSwitchGridLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85) + 1
        color: "white"
        border.width: 1
        border.color: "white"
        radius: coreObj.defaultLabelHeight * 0.5
        visible: (sensorsQml.humidityBuiltInSensorIsPresent || (sensorsQml.useSensorTag && sensorsQml.sensorTagConnected))
    }

    GridLayout {
        id: h2oSwitchGridLayout
        flow: GridLayout.LeftToRight
        anchors.left: frameRect.left
        anchors.bottom: frameRect.top
        visible: (sensorsQml.humidityBuiltInSensorIsPresent || (sensorsQml.useSensorTag && sensorsQml.sensorTagConnected))
        CustomLabel {
            text: qsTr("H2O")
            font.pointSize: font.pointSize + 5
        }

        CustomLabel {
            id: manualEntryLabel
            text: qsTr("Manual\nentry")
            MouseArea {
                anchors.fill: parent
                onClicked: h2oSwitch.checked = false
            }
        }
        CustomSwitch {
            id: h2oSwitch
            Binding on implicitWidth {
               when: (Screen.primaryOrientation !== Qt.PortraitOrientation)
               value: manualEntryLabel.width
            }
            padding: 0
            checked: sensorsQml.humiditySensorEnabled
            onCheckedChanged: {
                sensorsQml.humiditySensorEnabled = checked
                remoteControlQml.broadcastCommand("USEHUMID", 0)
            }
        }
        CustomLabel {
            id: sensorDataLabel
            text: qsTr("Sensor\nvalue")
            MouseArea {
                anchors.fill: parent
                onClicked: h2oSwitch.checked = true
            }
        }
    }

    // Column layout containing controls
    ColumnLayout {
        id: moleculesColumnLayout
        anchors.top: exitButton.bottom
        anchors.topMargin: coreObj.defaultLabelHeight * 0.95
        anchors.horizontalCenter: moleculesItem.horizontalCenter

        // Label containing the title for the current window
        CustomLabel {
            id: titleLabel
            anchors.top: moleculesColumnLayout.top
            text: "Concentration may be specified in %, parts  \nper million (ppmv), or as partial pressure"
        }      

        // Grid layout to contain information for all molecules (selected state, chemical formula, color to be used in the plot)
        // This list has to be populated during the program execution depending on the data in the configuration files or the internal information
        RowLayout {
            id: moleculesTitlesRowLayout
            anchors.top: titleLabel.bottom
            anchors.topMargin: 5

            CustomLabel {
                id: indexTitleLabel
                text: ((Qt.platform.os == 'winrt') || (Qt.platform.os == 'winphone') || (Qt.platform.os == 'windows') || (Qt.platform.os == 'linux')) ? " +" : "++"
            }

            // Label for the molecule name and checked state
            // Currently the program does not monitor the selection of all moleulces manually
            // (if all are unchecked or checked manually) then the select/deselect all
            // will not update, should be taken into account later
            CustomCheckBox {
                id: moleculeAllCheckboxTitle
                checked: molListViewModel.allSelected  // Need to change this to take into account if all are selected by default (currently all are selected by default)
                text: "Select All"
                onClicked: {
                    molListViewModel.selectAll(checked)
                }
            }

            // Combobox for the concentration units
            CustomComboBox {
                id: unitsComboBox
                model: [  "Concentration\n(%)", "Parts per\nmillion (ppmv)", "Atmospheres\n(atm)", "Millibar\n(mbar)", "Torr\n(Torr)", "Pascal\n(Pa)", "Pounds per\nsquare inch (psi)"]
                currentIndex:  molListViewModel.units
                implicitWidth: titleLabel.width - moleculeAllCheckboxTitle.width - indexTitleLabel.width
                Binding on font.pointSize {
                    when: (Qt.platform.os !== 'windows')
                    value: coreObj.defaultFontPointSize - 2
                }
                onCurrentIndexChanged:
                {
                    molListViewModel.units = currentIndex
                    totalPressQml.updateTotalPress()
                }
            }
        }

        // Listview with the list of molecules and their parameters
        ListView {
            id: molListView
            anchors.top: moleculesTitlesRowLayout.bottom
            anchors.topMargin: 5
            anchors.right: moleculesColumnLayout.right
            anchors.left: moleculesColumnLayout.left
            anchors.bottom: calcNotSelectedMolsRowLayout.top
            model: molListViewModel
            clip: true

            delegate: RowLayout {
                id: moleculeDelegate

                CustomLabel {
                    id: indexLabel
                    text: model.index
                }

                CustomCheckBox {
                    id: molSelNameCheckBox
                    // Had to do this here to overcome a reported bug (maybe has been fixed already)
                    // http://stackoverflow.com/questions/20552647/changing-a-model-of-a-qml-listview-doesnt-change-the-corresponding-delegate
                    Binding {
                        target: molSelNameCheckBox
                        property: 'checked'
                        value: model.checked
                    }
                    implicitWidth: moleculeAllCheckboxTitle.width
                    text: model.molName
                    onCheckStateChanged: { model.checked = checked
                        remoteControlQml.broadcastCommand("MSEL", model.index)
                    }
                }

                // Text field to change the partial pressure or concentration for the current molecule
                CustomTextField {
                    id: molConcentration
                    implicitWidth: titleLabel.width - moleculeAllCheckboxTitle.width - indexLabel.width - colorRect.width - 1.25 * moleculeDelegate.spacing
                    implicitHeight: refTextField.height * 0.92
                    text: model.pressValue
                    onEditingFinished: {
                        model.pressValue = text   // Used to update the value of the partial pressure or concentration in the model
                        remoteControlQml.broadcastCommand("MCONC", model.index)
                    }
                }

                // Rectanlge to contain the color corresponding to the current molecule to be used for plotting
                Rectangle {
                    id: colorRect
                    color: model.color
                    implicitWidth: moleculeDelegate.height * 0.85
                    implicitHeight: moleculeDelegate.height * 0.85
                    radius: 5

                    // Opening a color dialog for a given molecule
                    MouseArea {
                        anchors.fill: parent
                        onClicked:
                        {
                            molListViewModel.selectedColor = model.color
                            molListViewModel.activeIndex = model.index
                            var component = Qt.createComponent("ColorDialog.qml")
                            if (component.status === Component.Ready)
                            {
                                colorDialogWindow = component.createObject(moleculesItem)
                            }
                            colorDialogWindow.show()
                        }
                    }
                }
            }
        }

        RowLayout {
            id: calcNotSelectedMolsRowLayout
            anchors.bottom: moleculesColumnLayout.bottom
            anchors.horizontalCenter: parent.horizontalCenter            
            CustomLabel {
                id: onHumidityRowLabel
                anchors.right: useHumiditySensorSwitch.left
                anchors.verticalCenter: calcNotSelectedMolsSwitch.verticalCenter
                topPadding: coreObj.defaultLabelHeight * 0.4
                bottomPadding: coreObj.defaultLabelHeight * 0.1
                text: qsTr("Calculate for all")
                color: 'gray'
                MouseArea {
                    anchors.fill: parent
                    onClicked: calcNotSelectedMolsSwitch.checked = false
                }
            }
            CustomSwitch {
                id: calcNotSelectedMolsSwitch
                padding: 0
                topPadding: coreObj.defaultLabelHeight * 0.4
                bottomPadding: coreObj.defaultLabelHeight * 0.1
                checked: !molListViewModel.calcNotSelectedMols
                onCheckedChanged: {
                    molListViewModel.calcNotSelectedMols = !checked
                    remoteControlQml.broadcastCommand("ALLCALC", 0)
                }
            }
            CustomLabel {
                id: offHumidityRowLabel
                anchors.left: useHumiditySensorSwitch.right
                anchors.verticalCenter: calcNotSelectedMolsSwitch.verticalCenter
                topPadding: coreObj.defaultLabelHeight * 0.4
                bottomPadding: coreObj.defaultLabelHeight * 0.1
                text: qsTr("for selected only");
                color: 'black'
                MouseArea {
                    anchors.fill: parent
                    onClicked: calcNotSelectedMolsSwitch.checked = true
                }
            }
        }
    }
}
