// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// Interface to connect to a SensorTag device
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import QtQuick.Window 2.3

Item {

    id: sensorTagConnectItem
    anchors.fill: parent

    // This Javascript function is used to resize the connect item window
    // Resizing of form not done very nicely, need to figure out a better way in the future
    function resizeComponent() {
        if (Screen.primaryOrientation === Qt.PortraitOrientation) {
            mainConnectColumnLayout.height = sensorTagConnectItem.height - exitButton.height * 3.5; }
        else {
            mainConnectColumnLayout.height = sensorTagConnectItem.height + coreObj.defaultLabelHeight * (-1.7); }
    }

    // When the screen is rotated adjusting the position and orientation of the button used to display information about the communicaion protocol
    onHeightChanged:
    {
        // Resizing the connect window
        resizeComponent()
    }

    // Rectangle to shade the background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: formBorderRect
        anchors.fill: mainConnectColumnLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: sensorTagConnectItem.top
        anchors.right: sensorTagConnectItem.right
        anchors.margins: coreObj.defaultLabelHeight * 0.1
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            sensorTagConnectItem.destroy();
        }
    }

    CustomLabel {
        id: widthLabel
        text: qsTr("Setup connection to SensorTag over bluetooth\n(discover the SensorTag device then connect).")
        visible: false
        enabled: false
    }

    // Column layout to hold all the interface items
    ColumnLayout {
        id: mainConnectColumnLayout
        anchors.centerIn: parent
        anchors.horizontalCenter: sensorTagConnectItem.horizontalCenter
        anchors.top: (Screen.primaryOrientation === Qt.PortraitOrientation) ? exitButton.bottom : sensorTagConnectItem.top
        anchors.topMargin: coreObj.defaultLabelHeight * 1.2
        anchors.bottom: sensorTagConnectItem.bottom
        height: resizeComponent()

        CustomLabel {
            id: commentLabel
            anchors.top: mainConnectColumnLayout.top
            text: qsTr("Setup connection to SensorTag over bluetooth\n(discover the SensorTag device then connect).")
            onHeightChanged: resizeComponent()
        }

        // Bluetooth item to set the bluetooth connections
        ColumnLayout {
            id: bluetoothColumnLayout
            anchors.top: commentLabel.bottom
            anchors.bottom:  mainConnectColumnLayout.bottom
            anchors.topMargin: mainConnectColumnLayout.spacing

            Rectangle {
                id: bluetoothDevicesRect
                border.color: 'gray'
                anchors.top: bluetoothColumnLayout.top
                anchors.left: bluetoothColumnLayout.left
                anchors.right: bluetoothColumnLayout.right
                anchors.bottom: connectAndDiscoverButtonsLayout.top
                anchors.bottomMargin: mainConnectColumnLayout.spacing

                ListView {
                    id: bluetoothConnectListView
                    anchors.fill: parent
                    model: sensorTagBluetoothListModel
                    clip: true
                    currentIndex: sensorsQml.selectedDeviceIndex
                    onCurrentIndexChanged: sensorsQml.selectedDeviceIndex = currentIndex

                    delegate: RowLayout {
                        id: listViewRowItem
                        anchors.left: parent.left
                        anchors.right: parent.right

                        CustomLabel {
                            id: rowListLabel
                            text: model.deviceName
                            Layout.fillWidth: parent.width
                            padding: 10
                        }

                        // Mouse area to capture the selection event for a particular item in a list
                        MouseArea {
                            anchors.fill: parent
                            onClicked: bluetoothConnectListView.currentIndex = index
                        }
                    }                    

                    highlight:
                        Rectangle {
                        color: '#f5b7b1'
                        border.color: 'red'
                        border.width: 1
                    }
                }
            }

            RowLayout {
                id: connectAndDiscoverButtonsLayout
                anchors.bottomMargin: mainConnectColumnLayout.spacing
                anchors.bottom: useSensorTagRowLayout.top

                RowLayout {
                    id: buttonsRowLayout
                    anchors.centerIn: parent

                    CustomDialogButtonWithBackgroundSmall {
                        id: scanButton
                        implicitWidth: useSensorTagRowLayout.width - connectButton.width
                        text: qsTr("  Discover devices  ")
                        onClicked: sensorsQml.discoverDevices();
                    }
                    CustomDialogButtonWithBackgroundSmall {
                        id: connectButton
                        anchors.right: buttonsRowLayout.right
                        anchors.leftMargin: mainConnectColumnLayout.spacing
                        text: sensorsQml.connectButtonText
                        onClicked: sensorsQml.connectSensorTagDevice();
                    }
                }
            }

            RowLayout {
                id: useSensorTagRowLayout
                anchors.bottomMargin: mainConnectColumnLayout.spacing
                anchors.bottom: separatorRect.top

                CustomLabel {
                    id:  useSensorTagAuxLabel
                    text: qsTr(" Use SensorTag sensors: ")
                    MouseArea {
                        anchors.fill: parent
                        onClicked: useSensorTagSwitch.checked = !useSensorTagSwitch.checked
                    }
                }
                CustomLabel {
                    id: useSensorTagNoAuxLabel
                    text: qsTr("No")
                    color: 'gray'
                    MouseArea {
                        anchors.fill: parent
                        onClicked: sensorsQml.useSensorTag = false
                    }
                }
                CustomSwitch {
                    id: useSensorTagSwitch
                    padding: 0
                    checked: sensorsQml.useSensorTag
                    onCheckedChanged: {
                        sensorsQml.useSensorTag = checked
                        if (sensorsQml.sensorTagConnected) { remoteControlQml.broadcastCommand("USESENSTAG", 0) }
                    }
                }
                CustomLabel {
                    id: useSensorTagYesAuxLabel
                    text: qsTr("Yes")
                    color: 'red'
                    MouseArea {
                        anchors.fill: parent
                        onClicked: sensorsQml.useSensorTag = true
                    }
                }
            }

            Rectangle {
                id: separatorRect
                anchors.bottomMargin: mainConnectColumnLayout.spacing
                anchors.left: bluetoothColumnLayout.left
                anchors.right: bluetoothColumnLayout.right
                anchors.bottom: tempModeColumnLayout.top
                height: 1
                color: '#C0C0C0'
            }

            ColumnLayout {
                id: tempModeColumnLayout
                anchors.bottom: bluetoothColumnLayout.bottom

                // ComboBox to select the pathlength units
                CustomLabel {
                    id: tempUnitsLabel
                    text: qsTr("Sensor to use for temperature readings: ")
                }
                CustomComboBox {
                    id: unitsComboBox
                    model: [ "HDC1000 sensor", "BMP280 sensor", "HDC1000 & BMP280 sensors average"]
                    implicitWidth: bluetoothColumnLayout.width
                    currentIndex: sensorsQml.tempMode
                    onCurrentIndexChanged: sensorsQml.tempMode = currentIndex
                }
            }
        }
    }
}
