// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



// QML component is used to specify the vertical scale units
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3

Item {

    id: yAsisItem
    anchors.fill: parent

    // Rectangle to draw the shaded background
    Rectangle {
        anchors.fill: parent
        border.width: 1
        border.color: 'black'
        color: Qt.rgba(0, 0, 0, 0.6)

        // This is introduced to disable items located on the background
        MouseArea {
            anchors.fill: parent
            onClicked: { }
        }
    }

    // Rectangle to draw the form border
    Rectangle {
        id: formBorderRect
        anchors.fill: mainYAsisLayout
        anchors.margins: coreObj.defaultLabelHeight * (-0.85)
        color: "white"
        border.width: 1
        border.color: 'gray'
        radius: coreObj.defaultLabelHeight * 0.5
    }

    CustomLabel {
        id: widthLabel
        text: "The vertical axis scale and units"
        visible: false
        enabled: false
    }

    // Exit button
    CustomButton {
        id: exitButton
        anchors.top: yAsisItem.top
        anchors.right: yAsisItem.right
        anchors.margins: coreObj.defaultLabelHeight * (0.1)
        implicitWidth: 1.5 * exitButton.height
        text: qsTr("X")
        onClicked: {
            specResultsQml.updateYAxis();
            specResultsQml.updatePlotAllPreserveScale();
            yAsisItem.destroy();
        }
    }

    // Column layout to hold all the interface items
    ColumnLayout {

        // At rotation the buttons are not positioned exactly the way they should be (fix this)
        id: mainYAsisLayout
        anchors.centerIn: parent
        Layout.fillWidth: true
        Layout.fillHeight: true
        spacing: 3

        CustomLabel {
            id: commentLabel
            Layout.bottomMargin: coreObj.defaultLabelHeight * (0.2)
            Layout.fillWidth: true
            text: qsTr("  Vertical axis displayed value, scale and units  ")
        }

        TabBar {
            id: displayedParameterBar
            width: parent.width
            bottomPadding: coreObj.defaultLabelHeight * (0.15)
            currentIndex: yAxisQml.displayedValue
            onCurrentIndexChanged: {
                yAxisQml.displayedValue = currentIndex
                remoteControlQml.broadcastCommand("YPARAM", 0)
            }

            CustomTabButton {
                width: commentLabel.width * 0.5
                text: qsTr("Attenuation")
            }
            CustomTabButton {
                width: commentLabel.width * 0.49
                text: qsTr("Radiance")
            }
        }

        GridLayout {
            id: yAxisGridLayout
            rows: 3
            columns: 2

            // Minimum along the Y axis
            CustomLabel {
                id: yAxisMinimumLabel
                anchors.right: minimumTextField.left
                Layout.column: 0
                Layout.row: 0
                text: qsTr("  Minimum:  ")
            }
            CustomTextField {
                id: yAxisMinimumTextField
                Layout.fillWidth: true
                Layout.column: 1
                Layout.row: 0
                onEditingFinished: yAxisQml.minimum = text
                text: yAxisQml.minimum
                implicitWidth: widthLabel.width - minimumLabel.width
            }

            // Maximum along the Y axis
            CustomLabel {
                id: yAxisMaximumLabel
                anchors.right: maximumTextField.left
                Layout.column: 0
                Layout.row: 1
                text: qsTr("  Maximum:  ")
            }
            CustomTextField {
                id: yAxisMaximumTextField
                Layout.fillWidth: true
                Layout.column: 1
                Layout.row: 1
                text: yAxisQml.maximum
                onEditingFinished: yAxisQml.maximum = text
                implicitWidth: widthLabel.width - minimumLabel.width
            }
        }

        StackLayout {
            id: displayedParameterStackLayout
            currentIndex: displayedParameterBar.currentIndex

            // Vertical axis units
            CustomComboBox  {
                id: unitsComboBox
                Layout.fillWidth: true
                implicitWidth: widthLabel.width
                bottomPadding: 0
                model: ["Cross-section, cm^2 / molecule", "Optical depth", "Absorption, %", "Transmission, %" ]
                currentIndex: yAxisQml.units
                onCurrentIndexChanged: {
                    yAxisQml.units = currentIndex
                    remoteControlQml.broadcastCommand("YUNITS", 0)
                }
            }

            ColumnLayout {
                id: radiancePage               

                CustomCheckBox {
                    id: radianceAutoRecalculateCheckBox
                    bottomPadding: coreObj.defaultLabelHeight * (0.15)
                    text: qsTr(" Automatically recalculate max scale ")
                    checked: yAxisQml.radianceAutoRecalc
                    onCheckedChanged: {
                        yAxisQml.radianceAutoRecalc = checked
                        if (yAxisQml.displayedValue === 1)
                        {
                            specResultsQml.updateYAxis()
                            specResultsQml.updatePlotAllPreserveScale()
                            yAxisQml.updateScaleValues()
                        }
                    }
                }

                Rectangle {
                    id: separatorRectangle1
                    Layout.fillWidth: true
                    height: 1
                    color: '#D3D3D3'
                }
            }
        }

        CustomCheckBox {
            id: crossSectionAutoRecalculateCheckBox
            topPadding: coreObj.defaultLabelHeight * (0.25)
            bottomPadding: 0
            text: qsTr(" Automatically recalculate max scale ")
            checked: yAxisQml.crossSectionAutoRecalc
            onCheckedChanged: yAxisQml.crossSectionAutoRecalc = checked
            visible: ((yAxisQml.displayedValue === 0) && (yAxisQml.units === 0)) ? true : false
        }

        // This component enables or disables rulers shich track the intensity of the spectra for the wavenumber range center wavenumber
        CustomCheckBox {
            id: enableRullersCheckBox
            topPadding: coreObj.defaultLabelHeight * (0.3)
            bottomPadding: 0
            text: qsTr(" Enable rulers on zoom")
            checked: yAxisQml.enableRulers
            onCheckedChanged: yAxisQml.enableRulers = checked
        }
    }
}
