// bytran -|- spectral calculations for portable devices using the HITRAN database
//
// Developed by Dzianis Pliutau (Denis Plutov) and Kostya Roslyakov
// www.bytran.org | bytran@bytran.org
// Copyright (c) 2016 Dzianis Pliutau
// Distributed under the MIT license
//
// The MIT License (MIT)
// Copyright (c) 2016 Dzianis Pliutau
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
// is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
// IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


// Main QML window
import QtQuick 2.9
import QtQuick.Controls 2.2
import QtQuick.Layouts 1.3
import QtQuick.Dialogs 1.2
import QtQuick.Window 2.3

ApplicationWindow {

    id: mainWindow
    visible: true
    width: ((Qt.platform.os === 'windows') || (Qt.platform.os === 'linux')) ? 1000 : 640
    height: 480
    title: qsTr("bytran -|- spectral calculations for portable devices using the HITRAN database")

    // Dialog to exit an applicaion
    MessageDialog {
        id: exitDialog
        icon: StandardIcon.Question
        standardButtons: StandardButton.Yes | StandardButton.No
        function show() {
            exitDialog.text = "Would you like to exit the application?";
            exitDialog.open()
        }
        onYes: { coreObj.saveConfiguration(); Qt.quit(); }
        onNo: { exitDialog.close(); }
    }

    // Handing the case when the user presses the hardware back button to initiate the program close with a dialog
    onClosing: {
        close.accepted = false
        var component = Qt.createComponent("ExitDialog.qml")
        if (component.status === Component.Ready)
        {
            exitWindow = component.createObject(mainWindow)
        }
        exitWindow.show()
    }

    MainForm {
        id: mainFormItem
        anchors.fill: parent

        // Using the custom label here for iOS
        CustomSizingLabel {
            id: heightLabel
            text: qsTr("  Atmosphere model  ")
            visible: false
            enabled: false
            onWidthChanged: {

                // Saving the default height lavel parameter used to resize other controls
                coreObj.fontScalingRatio = (122.609375 / heightLabel.width)
                coreObj.defaultFontPointSize = ((Qt.platform.os === 'winrt') || (Qt.platform.os === 'winphone')) ? heightLabel.font.pointSize * coreObj.fontScalingRatio : heightLabel.font.pointSize * coreObj.fontScalingRatio  // For Windows Phone the scaling factor multiplication leads to erroneous results for some reason
                coreObj.defaultLabelHeight = ((Qt.platform.os === 'android') || (Qt.platform.os === 'winrt')) ? coreObj.fontScalingRatio * heightLabel.height : heightLabel.height  // This was adjusted like that to look good on other Android devices with different screen size ratios

                // At the initialization getting the height of the default label to adjust the interface
                // Plot axes titles font adjustment for different platforms
                if ((Qt.platform.os === 'winrt') || (Qt.platform.os === 'winphone')) { mainFormItem.spectrumPlot.fontPointSize = coreObj.defaultFontPointSize }
                else if (Qt.platform.os === 'windows') { mainFormItem.spectrumPlot.fontPointSize = coreObj.defaultFontPointSize + 2.5 }
                else if (Qt.platform.os === 'linux') { mainFormItem.spectrumPlot.fontPointSize = coreObj.defaultFontPointSize + 2.2 }
                else if ((Qt.platform.os === 'osx') || (Qt.platform.os === 'ios')) { mainFormItem.spectrumPlot.fontPointSize = coreObj.defaultFontPointSize + 1 + 5 * (1 - coreObj.fontScalingRatio) }
                                                                              else { mainFormItem.spectrumPlot.fontPointSize = coreObj.defaultFontPointSize + 5 * (1 - coreObj.fontScalingRatio) }

                // Saving the height of the screen to get the vertical resolution for ploting
                specResultsQml.heightPoints = mainFormItem.spectrumPlot.xHeight
                specResultsQml.updatePlotMolsOnlyPreserveScale()

                // Updating the reduced molecules list
                molListViewReducedModel.updateDisplayed();
            }
        }

        // On completed activating the notifier window
        Component.onCompleted: {

            // This had to be initialized here for the height of the item to be correct
            mainFormItem.totalsButtonRect.height = mainFormItem.menuButtonText.height * 1.9

            // Notification window is displayed when the height of the window is changed
            // Need to replace this with some kind of an on-load signal
            var component = Qt.createComponent("Notifier.qml")
            if (component.status === Component.Ready)
            {
                notifierWindow = component.createObject(mainWindow)
            }
            notifierWindow.show()
        }

        spectrumPlot.onHeightChanged: {
            // Saving the height of the screen to get the vertical resolution for ploting
            specResultsQml.heightPoints = spectrumPlot.xHeight
            specResultsQml.updatePlotMolsOnlyPreserveScale()
        }
        spectrumPlot.onPlotInProgressChanged: {
            coreObj.updatePlotInProgress(spectrumPlot.plotInProgress)
        }

        // Transfering the pinch gesture coordinates to the plot for zoom in / zoom out
        // Had to disable spectrum pan and plot features because they caused the program to crash if the display is touched while the line-by-line calculation is running
        plotPinchArea.onPinchStarted: {
            if (coreObj.calculationIsRunning() === false)
            {
                spectrumPlot.touchCenterX = pinch.center.x;
                spectrumPlot.touchCenterY = pinch.center.y;
                if (Math.abs(pinch.point1.x - pinch.point2.x) > Math.abs(pinch.point1.y - pinch.point2.y))
                     { spectrumPlot.isHorizontalPinch = true;  }
                else { spectrumPlot.isHorizontalPinch = false; }
            }
        }
        plotPinchArea.onPinchUpdated: {
            if (coreObj.calculationIsRunning() === false) { spectrumPlot.scale = pinch.scale; }
        }

        // Functionality to enable spectrum pan
        // Had to disable spectrum pan and plot features because they caused the program to crash if the display is touched while the line-by-line calculation is running
        plotMouseArea.onPressed: {
            if (coreObj.calculationIsRunning() === false) {
                spectrumPlot.touchStartX = mouse.x
                spectrumPlot.touchStartY = mouse.y
            }
        }
        plotMouseArea.onMouseXChanged: { if (coreObj.calculationIsRunning() === false) { spectrumPlot.touchEndX = mouse.x } }
        plotMouseArea.onMouseYChanged: { if (coreObj.calculationIsRunning() === false) { spectrumPlot.touchEndY = mouse.y } }

        // Changing from individual molecules to the total spectrum
        totalMolsButton.onCheckedChanged: {
            specResultsQml.showTotal = totalMolsButton.checked
            showTotalOptionsRectangle.visible = totalMolsButton.checked
            specResultsQml.updateYAxis()
            specResultsQml.updatePlotAllPreserveScale()
            remoteControlQml.broadcastCommand("TOTAL", 0)
        }

        // Opening the main menu
        menuButton.onClicked: {
            var component = Qt.createComponent("Menu.qml")
            if (component.status === Component.Ready)
            {
                menuWindow = component.createObject(mainWindow)
            }
            menuWindow.show()
        }

        // Running the calculation
        calcButton.onClicked: {
            specResultsQml.showPlainTotal = true;
            specResultsQml.showInstrTotal = true;
            remoteControlQml.broadcastCommand("CALC", 0)   // Broadcasting the calculate command (if the broadcasting mode is enabled)
            coreObj.calculate();
        }

        exitButton.onClicked: {
            var component = Qt.createComponent("ExitDialog.qml")
            if (component.status === Component.Ready)
            {
                exitWindow = component.createObject(mainWindow)
            }
            exitWindow.show()
        }

        // Opening a window to change the molecular parameters
        moleculesButton.onClicked: {
            var component = Qt.createComponent("Molecules.qml")
            if (component.status === Component.Ready)
            {
                moleculesWindow = component.createObject(mainWindow)
            }
            moleculesWindow.show()
        }

        // An update to the Ok dialog text will trigger opening of the Ok Dialog
        okDialogTextLabel.onTextChanged: {
            if (okDialogTextLabel.text !== "undefined")
            {
                var component = Qt.createComponent("OkDialog.qml")
                if (component.status === Component.Ready)
                {
                    okDialogWindow = component.createObject(mainWindow, {"message": coreObj.okDialogText})
                }
                okDialogWindow.show()
            }
        }

        // This section is to save the plot to a file
        spectrumPlot.onSaveImageChanged: {
            // This is done in order to add the information string at the top of the plot
            totalsButtonRect.height = 0
            spectrumPlot.addTopString()
            // Making buttons invisible for wen the image is grabbed
            menuButton.visible = false
            calcButton.visible = false
            moleculesButton.visible = false
            totalMolsButton.visible = false
            exitButton.visible = false
            plotAndMolViewRect.grabToImage(function(result) { result.saveToFile(spectrumPlot.saveImage); }, Qt.size(spectrumPlot.width * Screen.devicePixelRatio, spectrumPlot.height * Screen.devicePixelRatio));
        }
        spectrumPlot.onFileWritingCompleteChanged: {
            shareResultsQml.shareImageAndText();
            spectrumPlot.updatePlot()
            notifyQml.notification = ''
            notifyQml.visible = false
            totalsButtonRect.height = totalMolsButton.height * 1.15
            shareResultsQml.shareResults = false
            menuButton.visible = true
            calcButton.visible = true
            moleculesButton.visible = true
            totalMolsButton.visible = true
            exitButton.visible = true
        }

        // Updating the spectrum when switching between the Radiance and the attenuation spectrum
        showTotalOrRadianceButton.onCheckedChanged: {
            if (showTotalOrRadianceButton.checked) { yAxisQml.displayedValue = 1 }
                                              else { yAxisQml.displayedValue = 0 }
            specResultsQml.updateYAxis();
            specResultsQml.updatePlotAllPreserveScale()
        }
    }
}
